/**
 * Back 2 Browser Bytecode Translator
 * Copyright (C) 2012-2015 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://opensource.org/licenses/GPL-2.0.
 */
package org.apidesign.vm4brwsr;

import net.java.html.js.JavaScriptBody;
import net.java.html.lib.Date;
import static net.java.html.lib.Exports.NaN;
import static net.java.html.lib.Exports.eval;
import static net.java.html.lib.Exports.isNaN;
import static net.java.html.lib.Exports.parseInt;
import net.java.html.lib.Objs;

public class LibUse {
    public static int fourtyTwo() {
        Number n = (Number) eval("6 * 7");
        return n.intValue();
    }

    public static String hiProperty(String set) {
        final Object obj = eval("var x = {}; x.x = 'Hi'; x");
        if (obj == null) {
            throw new IllegalStateException("Some Value returned " + obj);
        }
        if (!(obj instanceof Objs)) {
            throw new IllegalStateException("The result is Objs: " + obj.getClass());
        }
        Objs js = (Objs) obj;
        if (set != null) {
            js.$set("x", set);
        }
        return (String) js.$get("x");
    }

    public static double parse(String str) throws Exception {
        return parseInt(str);
    }

    public static double parse(String str, int radix) throws Exception {
        return parseInt(str, radix);
    }

    public boolean checkNaN() throws Exception {
        double nan = NaN;
        return isNaN(nan);
    }

    public double dateOperation() throws Exception {
        Date eleven = new Date(2016, 1, 10);
        return eleven.getFullYear() + eleven.getMonth();
    }

    public static double arrayForEach() throws Exception {
        net.java.html.lib.Array<java.lang.Number> arr = new net.java.html.lib.Array<>();
        assertNumber(arr.length.get(), 0.0, "Empty at first");
        arr.push(1.1, 2.2, 3);
        assertNumber(arr.$get(0), 1.1, "1.1 is on position 0");
        assertNumber(arr.$get(1), 2.2, "2.2 is on position 1");
        assertNumber(arr.$get(2), 3.0, "3 is on position 2");
        final boolean[] called = { false };
        final double[] sum = { 0.0 };
        arr.forEach(new net.java.html.lib.Function.A1<java.lang.Number,Void>() {
            @Override
            public Void call(java.lang.Number p1) {
                called[0] = true;
                sum[0] += p1.doubleValue();
                return null;
            }

            @Override
            public Void call(Number p1, Object p2) {
                return call(p1);
            }

            @Override
            public Void call(Number p1, Object p2, Object p3) {
                return call(p1);
            }

            @Override
            public Void call(Number p1, Object p2, Object p3, Object p4) {
                return call(p1);
            }

            @Override
            public Void call(Number p1, Object p2, Object p3, Object p4, Object p5) {
                return call(p1);
            }
        });
        assertTrue(called[0], "forEach invoked");
        assertEquals(sum[0], 6.3, 0.1, "Sum computed");

        return sum[0];
    }

    private static void assertEquals(double real, double expected, double delta, String msg) {
        if (expected - delta < real && real < expected + delta) {
            return;
        }
        raise(msg + " exp: " + expected + " but was: " + real);
    }

    @JavaScriptBody(args = { "msg" }, body = "throw msg;")
    private static native void raise(String msg);

    private static void assertNumber(Object value, double expected, String msg) {
        if (value instanceof Number) {
            assertEquals(((Number)value).doubleValue(), expected, 0.1, msg);
        } else {
            if (value == null) {
                raise("null is not a number");
                return;
            }
            raise("Not a number: " + value + " type: " + value.getClass());
        }
    }

    private static void assertTrue(boolean b, String msg) {
        if (!b) {
            raise(msg);
        }
    }

}
